@@ -8,6 +8,7 @@
 #include <QXmlStreamAttributes>
 #include <QMessageBox>
 #include <QStandardPaths>
+#include <QProcessEnvironment>
 
 #include "MainUI/MainWindow.h"
 #include "MainUI/BookBrowser.h"
@@ -82,7 +83,7 @@ PluginRunner::~PluginRunner()
 QStringList PluginRunner::SupportedEngines()
 {
     QStringList engines;
-    engines << "python2.7" << "python3.4" << "python2.7,python3.4" << "python3.4,python2.7";
+    engines << "python3.4" << "python2.7,python3.4" << "python3.4,python2.7";
     return engines;
 }
 
@@ -138,7 +139,7 @@ int PluginRunner::exec(const QString &name)
     launcher_root = PluginDB::launcherRoot();
 
     // Note: Keep SupportedEngines() in sync with the engine calling code here.
-    if ( m_engine.contains("python2.7") || m_engine.contains("python3.4") ) {
+    if ( m_engine.contains("python3.4") ) {
         m_launcherPath = launcher_root + "/python/launcher.py";
         m_pluginPath = m_pluginsFolder + "/" + m_pluginName + "/" + "plugin.py";
         if (!QFileInfo(m_launcherPath).exists()) {
@@ -192,6 +193,9 @@ void PluginRunner::writeSigilCFG()
 {
     QStringList cfg = QStringList() << QCoreApplication::applicationDirPath();
     cfg << Utility::DefinePrefsDir();
+#if !defined(Q_OS_WIN32) && !defined(Q_OS_MAC)
+    cfg << Utility::LinuxHunspellDictionaryDirs().join(":");
+#endif
     QList <Resource *> selected_resources = m_bookBrowser->AllSelectedResources();
     foreach(Resource * resource, selected_resources) {
         cfg << resource->GetRelativePathToRoot();
@@ -247,6 +251,15 @@ void PluginRunner::startPlugin()
     args.append(QDir::toNativeSeparators(m_pluginPath));
     QString executable = QDir::toNativeSeparators(m_enginePath);
 
+#ifdef Q_OS_MAC
+    // On Mac OS X, it appears that QProcess does not inherit the callers process environment at all
+    // which directly contradicts the Qt documentation.
+    // So we simply read the system environment and set it for QProcess manually
+    // so that python getpreferredencoding() and stdout/stderr/stdin encodings to get properly set
+    QProcessEnvironment env = QProcessEnvironment::systemEnvironment();
+    m_process.setProcessEnvironment(env);
+#endif
+
     m_process.start(executable, args);
     ui.statusLbl->setText("Status: running");
 
@@ -480,15 +493,24 @@ bool PluginRunner::processResultXML()
                 }
 
                 QString linenumber;
-                bool   lok;
-                int vlinenumber;
+                bool   line_ok;
+                int vlinenumber = -1;
                 linenumber = attr.value("linenumber").toString();
-                vlinenumber = linenumber.toInt(&lok);
-                if (!lok) {
-                    vlinenumber = 0;
+                vlinenumber = linenumber.toInt(&line_ok);
+                if (!line_ok) {
+                    vlinenumber = -1;
                 }
 
-                m_validationResults.append(ValidationResult(vtype, attr.value("filename").toString(), vlinenumber<0?0:(size_t)vlinenumber, attr.value("message").toString()));
+                QString charoffset;
+                bool   coff_ok;
+                int vcharoffset = -1;
+                charoffset = attr.value("charoffset").toString();
+                vcharoffset = charoffset.toInt(&coff_ok);
+                if (!coff_ok) {
+                    vcharoffset = -1;
+                }
+
+                m_validationResults.append(ValidationResult(vtype, attr.value("filename").toString(), vlinenumber, vcharoffset, attr.value("message").toString()));
             }
         }
     }
