@@ -200,6 +200,11 @@ public:
 
     static void clearMemoryCaches();
 
+    /**
+     * Handle warning the user about undefined url fragments when splitting/merging resources.
+     */
+    bool ProceedWithUndefinedUrlFragments();
+
 public slots:
     void AnyCodeView();
 
@@ -648,6 +653,7 @@ private slots:
     void unloadPluginsMenu();
 
 private:
+    void updateToolTipsOnPluginIcons();
     void UpdateClipButton(int clip_number, QAction *ui_action);
     void InsertFiles(const QStringList &selected_images);
     void InsertFilesFromDisk();
