@@ -45,7 +45,8 @@
 #include <QtCore/QUrl>
 #include <QtCore/QUuid>
 #include <QtWidgets/QMainWindow>
-#include <QtWidgets/QMessageBox>
+#include <QTextEdit>
+#include <QMessageBox>
 #include <QRegularExpression>
 #include <QRegularExpressionMatch>
 #include <QFile>
@@ -54,6 +55,7 @@
 #include "sigil_constants.h"
 #include "sigil_exception.h"
 #include "Misc/QCodePage437Codec.h"
+#include "Misc/SettingsStore.h"
 
 #ifndef MAX_PATH
 // Set Max length to 256 because that's the max path size on many systems.
@@ -64,6 +66,24 @@
 
 static QCodePage437Codec *cp437 = 0;
 
+// Subclass QMessageBox for our StdWarningDialog to make any Details Resizable
+class SigilMessageBox: public QMessageBox
+{
+    public:
+        SigilMessageBox(QWidget* parent) : QMessageBox(parent) 
+        {
+            setSizeGripEnabled(true);
+        }
+    private:
+        virtual void resizeEvent(QResizeEvent * e) {
+            QMessageBox::resizeEvent(e);
+            setMaximumSize(QWIDGETSIZE_MAX, QWIDGETSIZE_MAX);
+            if (QWidget *textEdit = findChild<QTextEdit *>()) {
+                textEdit->setMaximumHeight(QWIDGETSIZE_MAX);
+            }
+        }
+};
+
 #include "Misc/Utility.h"
 
 
@@ -80,6 +100,42 @@ QString Utility::DefinePrefsDir()
 }
 
 
+#if !defined(Q_OS_WIN32) && !defined(Q_OS_MAC)
+// Return correct path(s) for Linux hunspell dictionaries
+QStringList Utility::LinuxHunspellDictionaryDirs()
+{
+    QStringList paths;
+    // prefer the directory specified by the env var SIGIL_DICTIONARIES above all else.
+    if (!hunspell_dicts_override.isEmpty()) {
+        // Handle multiple colon-delimited paths
+        foreach (QString s, hunspell_dicts_override.split(":")) {
+            paths << s.trimmed();
+        }
+    }
+    // else use the env var runtime overridden 'share/sigil/hunspell_dictionaries/' location.
+    else if (!sigil_extra_root.isEmpty()) {
+        paths.append(sigil_extra_root + "/hunspell_dictionaries/");
+    }
+    // Bundled dicts were not installed use standard system dictionary location.
+    else if (!dicts_are_bundled) {
+        paths.append("/usr/share/hunspell");
+        // Add additional hunspell dictionary directories. Provided at compile
+        // time via the cmake option EXTRA_DICT_DIRS (colon separated list).
+        if (!extra_dict_dirs.isEmpty()) {
+            foreach (QString s, extra_dict_dirs.split(":")) {
+                paths << s.trimmed();
+            }
+        }
+    }
+    else {
+        // else use the standard build time 'share/sigil/hunspell_dictionaries/'location.
+        paths.append(sigil_share_root + "/hunspell_dictionaries/");
+    }
+    return paths;
+}
+#endif
+
+
 // Uses QUuid to generate a random UUID but also removes
 // the curly braces that QUuid::createUuid() adds
 QString Utility::CreateUUID()
@@ -337,24 +393,8 @@ bool Utility::RenameFile(const QString &oldfilepath, const QString &newfilepath)
 
 QString Utility::GetTemporaryFileNameWithExtension(const QString &extension)
 {
-    return QDir::temp().absolutePath() + "/sigil_" + Utility::CreateUUID() + extension;
-}
-
-// Creates a copy of the provided file with a random name in
-// the systems TEMP directory and returns the full path to the new file.
-// The extension of the original file is preserved. If the original file
-// doesn't exist, an empty string is returned.
-QString Utility::CreateTemporaryCopy(const QString &fullfilepath)
-{
-    if (!QFileInfo(fullfilepath).exists() || !QFileInfo(fullfilepath).isFile()) {
-        return QString();
-    }
-
-    QString temp_file_path = QDir::temp().absolutePath() + "/" +
-                             Utility::CreateUUID() + "." +
-                             QFileInfo(fullfilepath).suffix();
-    QFile::copy(fullfilepath, temp_file_path);
-    return temp_file_path;
+    SettingsStore ss;
+    return ss.tempFolderHome() +  "/sigil_" + Utility::CreateUUID() + extension;
 }
 
 
@@ -528,7 +568,7 @@ void Utility::DisplayStdErrorDialog(const QString &error_message, const QString
 
 void Utility::DisplayStdWarningDialog(const QString &warning_message, const QString &detailed_text)
 {
-    QMessageBox message_box(QApplication::activeWindow());
+    SigilMessageBox message_box(QApplication::activeWindow());
     message_box.setWindowFlags(Qt::Window | Qt::WindowStaysOnTopHint);
     message_box.setModal(true);
     message_box.setIcon(QMessageBox::Warning);
@@ -539,7 +579,6 @@ void Utility::DisplayStdWarningDialog(const QString &warning_message, const QStr
     if (!detailed_text.isEmpty()) {
         message_box.setDetailedText(detailed_text);
     }
-
     message_box.setStandardButtons(QMessageBox::Close);
     message_box.exec();
 }
