@@ -523,6 +523,16 @@ QString NavProcessor::GetLandmarkCodeForResource(const Resource *resource)
     return etype;
 }
 
+QString NavProcessor::GetLandmarkNameForResource(const Resource *resource)
+{
+    QString name;
+    QString etype = GetLandmarkCodeForResource(resource);
+    if (!etype.isEmpty()) {
+        name = Landmarks::instance()->GetName(etype);
+    }
+    return name;
+}
+
 QHash <QString, QString> NavProcessor::GetLandmarkNameForPaths()
 {
     const QList<NavLandmarkEntry> landlist = GetLandmarks();
@@ -531,8 +541,8 @@ QHash <QString, QString> NavProcessor::GetLandmarkNameForPaths()
     foreach(NavLandmarkEntry le, landlist) {
         QString href = ConvertHREFToOEBPSRelative(le.href);
         QStringList parts = href.split('#', QString::KeepEmptyParts);
-        QString title = le.title;
-        semantic_types[parts.at(0)] = title;
+        QString etype = le.etype;
+        semantic_types[parts.at(0)] = Landmarks::instance()->GetName(etype);
     }
     return semantic_types;
 }
